function [amp,f,g,amax,amin,fp,f3db,psi,phase]=bkfil_e(N,rx0,Fa)
% bkfil_e	EMQF (Elliptic Minimal Q Factors) digital filter design.
%		[amp,f,g,amax,amin,Fp,F3db,psi]=bkfil_e(N,rx0,Fa) designs 
%       	an N'th order lowpass digital EMQF filter with normalized 
%               stopband edge frequency Fa. The sampling frequency is assumed to be Fs=1.
%               The constant rx0 is chosen by designer and represents rx0=1/x0
%               where x0 is a real number defined as the center of the circle, 
%               the poles loci in the z plane.
%		For given Fp and Fa (Fp and Fa the boundary frequencies of the specificatin),
%               the permited values rx0 are in the range:
%
%               (1-(tan(pi*Fa)^2)/(1+(tan(pi*Fa)^2)<rx0<(1-(tan(pi*Fp)^2)/(1+(tan(pi*Fp)^2).
%
%	   The value of rx0 approximates 
%              
%               (1-(tan(pi*Fa)(tan(pi*Fp))/(1+(tan(pi*Fa)(tan(pi*Fp))
% 
%               and is adjusted to be represented as a simple combination of powers of two. 
%    
%     In the case of a half-band filter rxo=0         
%
%       bkfil_e returns:
%              * Amplitude response in vector amp, length 500. 
%              * Normalized frequencies (Fs=1) in vector f, length 500. 
%              * z plane poles in vector g. Vector g containes the poles
%                of the upper halfplane orderd by the increasing magnitude.
%              * passband ripple amax (dB)
%              * minimal stopband attenuation amin
%              * passband boundary frequency of EMQF filter fp
%              * 3 dB attenuation frequency f3db
%              * phase difference function of allpass realization in vector psi, length 500.
%                psi=(fia-fib)/2, fia phase of branch a, fib phase of branch b
% 
% Required M files: 
% ellc2, ellc3, ellca, ellz, elll0p5, elll2, elll3, ellll, ellllst,
% ellm, elln,ellr2r3r, ellt, ellz, ellz30, ellz33, ellz4	
f3db=(acos(rx0))/(2*pi);
M=0.5/f3db;
omega=(tan(pi*Fa)/tan(pi/(2*M)))^2;
Fa=atan(sqrt(omega)*tan(pi/(2*M)))/pi;
fp=atan(tan(pi/(2*M))/sqrt(omega))/pi;
L=ellll(N,omega);
Eps=1/sqrt(L);
amax=-10*log10(1/(1+Eps^2));
amin=-10*log10(1/(1+L));
[zzd,ppd,co]=ellip(N,amax,amin,2*fp);
 
ppd=sort(ppd);	% sort in moduli increasing order   

l=0;	% sort nonnegative imaginary part poles
for k=1:N
if imag(ppd(k))>=0
l=l+1;
g(l)=ppd(k);
else
end
end

aa=g(1:2:(N+1)/2);
aa=sort(cplxpair([aa,conj(aa(2:length(aa)))])); % poles of the branch a
bb=g(2:2:(N+1)/2);
bb=cplxpair([bb,conj(bb)]); % poles of the branch b
Hab=real(poly(aa));
Hai=fliplr(Hab);
Hbb=real(poly(bb));
Hbi=fliplr(Hbb);

[Ha,w]=freqz(Hab,Hai,2000);
[Hb,w]=freqz(Hbb,Hbi,2000);
fia=unwrap(angle(Ha));
fib=unwrap(angle(Hb));
psi=(fia-fib)/2;
phase=fia+fib;
amp=cos(psi);
f=w/(2*pi);



