% ############################################################################
%
% ellm.m    Elliptic or Cauer filter design.
%           [C,Z,Wp,Q_factor]=ellm(N,Ws,Eps) designs an Nth order lowpass
%           elliptic filter with
%           Ws  : normalized stopband edge frequency (unity cutoff frequency)
%           Eps : ripple - factor
%           1 < N (filter order) < 19
%           ellm.m returns
%           Z        : transfer function zeros are 0 +- j Z
%           C        : transfer function poles are C and conjugate of C
%           Wp       : magnitude of C
%           Q_factor : Q-factor of poles
%
%           M. Lutovac 6-18-1994
%           Reference: M. D. Lutovac and D. M. Rabrenovic', Determination of 
%           the natural modes of some Cauer filters without elliptic functions
%
% function [C,Z,Wp,Q_factor]=ellm(N,Ws,Eps)
% ____________________________________________________________________________

function [C,Z,Wp,Q_factor]=ellm(N,Ws,Eps)

if     N==2
   T   =sqrt(1-1/Ws^2);
   W  =Ws*sqrt(1-T);       Z   =Ws/W;
   z  =ellc2(Ws,Eps);      C   =polzeta(W,Ws,z);
elseif N==3
   W  =ellz30(Ws);         Z   =Ws/W;                W =[0 W];
   z  =ellc3(Ws,Eps);      C   =polzeta(W,Ws,z);
elseif N==4
   T   =sqrt(1-1/Ws^2);    Ws2=elll2(Ws);
   W  =Ws*sqrt(1-sqrt(T))*[sqrt(1+T-sqrt(T+T^2)) sqrt(1+T+sqrt(T+T^2))];
   Z  =Ws*ones(size(W))./W;
   z  =ellc2(Ws2,Eps);     Eps2=sqrt(1/z/z-1);
   z  =ellc2(Ws,Eps2);     C   =polzeta(W,Ws,z);
elseif N==6
   Ws3=elll3(Ws);          Ws2 =elll2(Ws);
   W  =ellz30(Ws2);        T   =sqrt(1-1/Ws^2);
   W   =ellz2(W,Ws);       W(3)=Ws*sqrt(1-T);
   W  =sort(W);            Z   =Ws*ones(size(W))./W;
   z  =ellc2(Ws3,Eps);     Eps2=sqrt(1/z/z-1);
   z  =ellc3(Ws,Eps2);      C  =polzeta(W,Ws,z);
elseif N==8
   Ws2=elll2(Ws);          Ws4 =elll2(Ws2);   
   W  =ellz4(Ws2);         W   =ellz2(W,Ws);
   W  =sort(W);            Z   =Ws*ones(size(W))./W;
   z  =ellc2(Ws4,Eps);     Eps2=sqrt(1/z/z-1);
   z  =ellc2(Ws2,Eps2);    Eps4=sqrt(1/z/z-1);
   z  =ellc2(Ws ,Eps4);    C   =polzeta(W,Ws,z);
elseif N==9
   Ws3=elll3(Ws);          W   =ellz30(Ws3);
   W  =ellz33(W,Ws);       W   =sort(W);
   Z  =Ws*ones(size(W))./W;      W   =[0 W];
   z  =ellc3(Ws3,Eps);     Eps3=sqrt(1/z/z-1);
   z  =ellc3(Ws ,Eps3);    C   =polzeta(W,Ws,z);
elseif N==12
   Ws3=elll3(Ws);          Ws6 =elll2(Ws3);
   W  =ellz4(Ws3);         W   =ellz33(W,Ws);
   W(8)=[];                W(4)=[];
   W  =sort(W);            Z   =Ws*ones(size(W))./W;
   z  =ellc2(Ws6,Eps);     Eps2=sqrt(1/z/z-1);
   z  =ellc2(Ws3,Eps2);    Eps4=sqrt(1/z/z-1);
   z  =ellc3(Ws,Eps4);   C   =polzeta(W,Ws,z);
elseif N==16
   Ws2=elll2(Ws);          Ws4 =elll2(Ws2);        Ws8=elll2(Ws4);
   W  =ellz4(Ws4);         W   =ellz2(W,Ws2);      W  =ellz2(W,Ws);
   W  =sort(W);            Z   =Ws*ones(size(W))./W;
   z  =ellc2(Ws8,Eps);     Eps2=sqrt(1/z/z-1);
   z  =ellc2(Ws4,Eps2);    Eps4=sqrt(1/z/z-1);
   z  =ellc2(Ws2,Eps4);    Eps8=sqrt(1/z/z-1);
   z  =ellc2(Ws,Eps8);     C   =polzeta(W,Ws,z);
elseif N==18
   T   =sqrt(1-1/Ws^2);
   Ws2=elll2(Ws);          Ws3 =elll3(Ws);
   Ws6=elll2(Ws3);         Ws9 =elll3(Ws3);
   W  =ellz30(Ws6);        W   =ellz33(W,Ws2);
   W  =ellz2(W,Ws);        W   =[W Ws*sqrt(1-T)];
   W  =sort(W);            Z   =Ws*ones(size(W))./W;
   z  =ellc2(Ws9,Eps);     Eps2=sqrt(1/z/z-1);
   z  =ellc3(Ws3,Eps2);  Eps6=sqrt(1/z/z-1);
   z  =ellc3(Ws,Eps6);   C   =polzeta(W,Ws,z);
else
   [R2,R3,Rest]=ellr2r3r(N);
   if Rest==1
      indW=1;WS(indW)=Ws;
      if R2>0
         for indR=1:R2
            indW=indW+1;  WS(indW)=elll2(WS(indW-1));
         end
      end
      if R3>0
         for indR=1:R3
            indW=indW+1;  WS(indW)=elll3(WS(indW-1));
         end
      end
      indw=indW-1;
      if R3>0
         W=ellz30(WS(indw));
         z=ellc3(WS(indw),EPS);  EPS=sqrt(1/z/z-1);     indw=indw-1;
         if R3>1
            for indR=2:R3
               W=ellz33(W,WS(indw));
               z=ellc3(WS(indw),EPS);  EPS=sqrt(1/z/z-1);    indw=indw-1;
            end
         end
      end
      if R2>0
         if R3==0
            W=WS(R2)*sqrt(1-sqrt(1-1/WS(R2)^2));  
            z  =ellc2(WS(R2),EPS);         EPS=sqrt(1/z/z-1);
            if R2>1
               indw=R2-1;
               for indR=1:R2-1
                  W=ellz2(W,WS(indw));
                  z  =ellc2(WS(indw),EPS);  EPS=sqrt(1/z/z-1);  indw=indw-1;
               end
            end
         else
            for indR=1:R2
               W=ellz2(W,WS(indw));
               z  =ellc2(WS(indw),EPS);  EPS=sqrt(1/z/z-1);  indw=indw-1;
            end
         end
      end
      if R2==0&R3>0
         Z =sort(Ws*ones(size(W))./W);       W=sort([0 W]);
      elseif R2>0&R3>0
         W1=WS(R2)*sqrt(1-sqrt(1-1/WS(R2)^2));  
         if R2>1
            indw=R2-1;
            for indR=1:R2-1
               W1=ellz2(W1,WS(indw));  indw=indw-1;
            end
         end
         W=sort([W1 W]);               Z=sort(Ws*ones(size(W))./W);
      elseif R2>0&R3==0
         W=sort(W);                    Z=sort(Ws*ones(size(W))./W);
      end
      C   =sort(polzeta(W,Ws,z));
   else
      if N==5 | N==7 | N==10 | N==11 | N==13 | N==14 | N==15 | N==17
         z=ellca(N,Ws,Eps);
         W=sort(ellz(N,Ws));  Z=Ws*ones(size(W))./W;
         if even(N)==1
            W=[0 W];
         end
         C=polzeta(W,Ws,z);
      else
      disp('  N > 18    i     N nije 2^i 3^j ')
   end
end
end
Wp=sqrt(real(C).^2+imag(C).^2);  Q_factor=-Wp./real(C)/2;
% ****************************************************************************
